<?php
/**
 * A general registration class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.3.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.mxchange.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseRegistration extends BaseFrameworkSystem implements UserRegister {
	/**
	 * Instance of a request class
	 */
	private $requestInstance = null;

	/**
	 * Instance of a response class
	 */
	private $responseInstance = null;

	/**
	 * Pre-registration filter chain
	 */
	private $preRegistrationFilter = null;

	/**
	 * Pre-registration filter chain
	 */
	private $postRegistrationFilter = null;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Initialize filters. This must be done before you can use them
	 *
	 * @return	void
	 */
	protected function initFilterChains () {
		// Pre/post-registration filters
		$this->preRegistrationFilter = ObjectFactory::createObjectByConfiguredName('filter_chain');
		$this->postRegistrationFilter = ObjectFactory::createObjectByConfiguredName('filter_chain');
	}

	/**
	 * Setter for request instance
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	void
	 */
	public final function setRequestInstance (Requestable $requestInstance) {
		$this->requestInstance = $requestInstance;
	}

	/**
	 * Getter for request instance
	 *
	 * @return	$requestInstance	An instance of a Requestable class
	 */
	public final function getRequestInstance () {
		return $this->requestInstance;
	}

	/**
	 * Setter for response instance
	 *
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	public final function setResponseInstance (Responseable $responseInstance) {
		$this->responseInstance = $responseInstance;
	}

	/**
	 * Getter for response instance
	 *
	 * @return	$responseInstance	An instance of a Responseable class
	 */
	public final function getResponseInstance () {
		return $this->responseInstance;
	}

	/**
	 * Adds a filter to the pre filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPreFilter (Filterable $filterInstance) {
		// Add the pre filter
		$this->preRegistrationFilter->addFilter($filterInstance);
	}

	/**
	 * Adds a filter to the post filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPostFilter (Filterable $filterInstance) {
		// Add the post filter
		$this->postRegistrationFilter->addFilter($filterInstance);
	}

	/**
	 * Executes all pre filters
	 *
	 * @return	void
	 */
	protected function executePreFilters () {
		// Execute all pre filters
		$this->preRegistrationFilter->processFilters($this->getRequestInstance(), $this->getResponseInstance());
	}

	/**
	 * Executes all post filters
	 *
	 * @return	void
	 */
	protected function executePostFilters () {
		// Execute all post filters
		$this->postRegistrationFilter->processFilters($this->requestInstance, $this->responseInstance);
	}
}

// [EOF]
?>
